#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# SPDX-License-Identifier: Apache-2.0
# Copyright 2022 Stéphane Caron

"""List of all robot descriptions."""

from dataclasses import dataclass
from enum import IntEnum
from typing import Dict, Set


class Format(IntEnum):
    """Format of a robot description."""

    URDF = 0
    MJCF = 1


@dataclass
class Description:
    """Metadata for a robot description.

    Attributes:
        formats: List of formats (URDF, MJCF) provided by the description.
    """

    formats: Set[Format]
    tags: Set[str]

    def __init__(self, single_format: Format, tags: Set[str] = set()):
        """Initialize a description that provides a single format.

        Args:
            single_format: Format provided by the description.
            tags: Set of strings describing properties of the description.
        """
        self.formats = {single_format}
        self.tags = tags

    @property
    def has_mjcf(self) -> bool:
        """Check if description provides MJCF."""
        return Format.MJCF in self.formats

    @property
    def has_urdf(self) -> bool:
        """Check if description provides URDF."""
        return Format.URDF in self.formats


DESCRIPTIONS: Dict[str, Description] = {
    "a1_description": Description(Format.URDF, tags={"quadruped"}),
    "a1_mj_description": Description(Format.MJCF, tags={"quadruped"}),
    "ability_hand_description": Description(
        Format.URDF, tags={"end_effector"}
    ),
    "ability_hand_mj_description": Description(
        Format.MJCF, tags={"end_effector"}
    ),
    "adam_lite_mj_description": Description(Format.MJCF, tags={"humanoid"}),
    "aliengo_description": Description(Format.URDF, tags={"quadruped"}),
    "aliengo_mj_description": Description(Format.MJCF, tags={"quadruped"}),
    "allegro_hand_description": Description(
        Format.URDF, tags={"end_effector"}
    ),
    "allegro_hand_mj_description": Description(
        Format.MJCF, tags={"end_effector"}
    ),
    "aloha_mj_description": Description(Format.MJCF, tags={"dual_arm"}),
    "anymal_b_description": Description(Format.URDF, tags={"quadruped"}),
    "anymal_b_mj_description": Description(Format.MJCF, tags={"quadruped"}),
    "anymal_c_description": Description(Format.URDF, tags={"quadruped"}),
    "anymal_c_mj_description": Description(Format.MJCF, tags={"quadruped"}),
    "anymal_d_description": Description(Format.URDF, tags={"quadruped"}),
    "apollo_mj_description": Description(Format.MJCF, tags={"humanoid"}),
    "arx_l5_mj_description": Description(Format.MJCF, tags={"arm"}),
    "atlas_drc_description": Description(Format.URDF, tags={"humanoid"}),
    "atlas_v4_description": Description(Format.URDF, tags={"humanoid"}),
    "b1_description": Description(Format.URDF, tags={"quadruped"}),
    "b2_description": Description(Format.URDF, tags={"quadruped"}),
    "bambot_description": Description(
        Format.URDF, tags={"mobile_manipulator"}
    ),
    "barrett_hand_description": Description(
        Format.URDF, tags={"end_effector"}
    ),
    "baxter_description": Description(Format.URDF, tags={"dual_arm"}),
    "berkeley_humanoid_description": Description(
        Format.URDF, tags={"humanoid"}
    ),
    "bolt_description": Description(Format.URDF, tags={"biped"}),
    "booster_t1_description": Description(Format.URDF, tags={"humanoid"}),
    "booster_t1_mj_description": Description(Format.MJCF, tags={"humanoid"}),
    "cassie_description": Description(Format.URDF, tags={"biped"}),
    "cassie_mj_description": Description(Format.MJCF, tags={"biped"}),
    "cf2_description": Description(Format.URDF, tags={"drone"}),
    "cf2_mj_description": Description(Format.MJCF, tags={"drone"}),
    "double_pendulum_description": Description(
        Format.URDF, tags={"educational"}
    ),
    "dynamixel_2r_mj_description": Description(
        Format.MJCF, tags={"educational"}
    ),
    "draco3_description": Description(Format.URDF, tags={"humanoid"}),
    "edo_description": Description(Format.URDF, tags={"arm"}),
    "elf2_description": Description(Format.URDF, tags={"humanoid"}),
    "elf2_mj_description": Description(Format.MJCF, tags={"humanoid"}),
    "ergocub_description": Description(Format.URDF, tags={"humanoid"}),
    "eve_r3_description": Description(
        Format.URDF, tags={"mobile_manipulator"}
    ),
    "fanuc_m710ic_description": Description(Format.URDF, tags={"arm"}),
    "fetch_description": Description(Format.URDF, tags={"mobile_manipulator"}),
    "finger_edu_description": Description(Format.URDF, tags={"educational"}),
    "fr3_mj_description": Description(Format.MJCF, tags={"arm"}),
    "g1_description": Description(Format.URDF, tags={"humanoid"}),
    "g1_mj_description": Description(Format.MJCF, tags={"humanoid"}),
    "gen2_description": Description(Format.URDF, tags={"arm"}),
    "gen3_description": Description(Format.URDF, tags={"arm"}),
    "gen3_lite_description": Description(Format.URDF, tags={"arm"}),
    "gen3_mj_description": Description(Format.MJCF, tags={"arm"}),
    "ginger_description": Description(
        Format.URDF, tags={"mobile_manipulator"}
    ),
    "go1_description": Description(Format.URDF, tags={"quadruped"}),
    "go1_mj_description": Description(Format.MJCF, tags={"quadruped"}),
    "go2_description": Description(Format.URDF, tags={"quadruped"}),
    "go2_mj_description": Description(Format.MJCF, tags={"quadruped"}),
    "gr1_description": Description(Format.URDF, tags={"humanoid"}),
    "h1_description": Description(Format.URDF, tags={"humanoid"}),
    "h1_mj_description": Description(Format.MJCF, tags={"humanoid"}),
    "h1_2_description": Description(Format.URDF, tags={"humanoid"}),
    "h1_2_mj_description": Description(Format.MJCF, tags={"humanoid"}),
    "hyq_description": Description(Format.URDF, tags={"quadruped"}),
    "icub_description": Description(Format.URDF, tags={"humanoid"}),
    "iiwa7_description": Description(Format.URDF, tags={"arm"}),
    "iiwa14_description": Description(Format.URDF, tags={"arm"}),
    "iiwa14_mj_description": Description(Format.MJCF, tags={"arm"}),
    "jaxon_description": Description(Format.URDF, tags={"humanoid"}),
    "jvrc_description": Description(Format.URDF, tags={"humanoid"}),
    "jvrc_mj_description": Description(Format.MJCF, tags={"humanoid"}),
    "laikago_description": Description(Format.URDF, tags={"quadruped"}),
    "leap_hand_v1_description": Description(
        Format.URDF, tags={"end_effector"}
    ),
    "leap_hand_mj_description": Description(
        Format.MJCF, tags={"end_effector"}
    ),
    "low_cost_robot_arm_mj_description": Description(
        Format.MJCF, tags={"arm"}
    ),
    "mini_cheetah_description": Description(Format.URDF, tags={"quadruped"}),
    "minitaur_description": Description(Format.URDF, tags={"quadruped"}),
    "mujoco_humanoid_mj_description": Description(
        Format.MJCF, tags={"educational", "humanoid"}
    ),
    "n1_description": Description(Format.URDF, tags={"humanoid"}),
    "n1_mj_description": Description(Format.MJCF, tags={"humanoid"}),
    "nextage_description": Description(Format.URDF, tags={"dual_arm"}),
    "op3_mj_description": Description(Format.MJCF, tags={"humanoid"}),
    "panda_description": Description(Format.URDF, tags={"arm"}),
    "panda_mj_description": Description(Format.MJCF, tags={"arm"}),
    "pepper_description": Description(
        Format.URDF, tags={"mobile_manipulator"}
    ),
    "piper_description": Description(Format.URDF, tags={"arm"}),
    "piper_mj_description": Description(Format.MJCF, tags={"arm"}),
    "poppy_ergo_jr_description": Description(Format.URDF, tags={"arm"}),
    "poppy_torso_description": Description(Format.URDF, tags={"dual_arm"}),
    "pr2_description": Description(
        Format.URDF, tags={"dual_arm", "mobile_manipulator"}
    ),
    "r2_description": Description(Format.URDF, tags={"humanoid"}),
    "rby1_description": Description(Format.URDF, tags={"mobile_manipulator"}),
    "reachy_description": Description(
        Format.URDF, tags={"mobile_manipulator"}
    ),
    "rhea_description": Description(Format.URDF, tags={"biped"}),
    "robotiq_2f85_description": Description(
        Format.URDF, tags={"end_effector"}
    ),
    "robotiq_2f85_mj_description": Description(
        Format.MJCF, tags={"end_effector"}
    ),
    "robotiq_2f85_v4_mj_description": Description(
        Format.MJCF, tags={"end_effector"}
    ),
    "romeo_description": Description(Format.URDF, tags={"humanoid"}),
    "rsk_description": Description(Format.URDF, tags={"wheeled"}),
    "rsk_mj_description": Description(Format.MJCF, tags={"wheeled"}),
    "sawyer_mj_description": Description(Format.MJCF, tags={"arm"}),
    "shadow_dexee_mj_description": Description(
        Format.MJCF, tags={"end_effector"}
    ),
    "shadow_hand_mj_description": Description(
        Format.MJCF, tags={"end_effector"}
    ),
    "sigmaban_description": Description(Format.URDF, tags={"humanoid"}),
    "simple_humanoid_description": Description(
        Format.URDF, tags={"humanoid", "educational"}
    ),
    "skydio_x2_description": Description(Format.URDF, tags={"drone"}),
    "skydio_x2_mj_description": Description(Format.MJCF, tags={"drone"}),
    "so_arm100_description": Description(Format.URDF, tags={"arm"}),
    "so_arm100_mj_description": Description(Format.MJCF, tags={"arm"}),
    "so_arm101_description": Description(Format.URDF, tags={"arm"}),
    "so_arm101_mj_description": Description(Format.MJCF, tags={"arm"}),
    "solo_description": Description(Format.URDF, tags={"quadruped"}),
    "spot_mj_description": Description(Format.MJCF, tags={"quadruped"}),
    "spryped_description": Description(Format.URDF, tags={"biped"}),
    "stretch_description": Description(
        Format.URDF, tags={"mobile_manipulator"}
    ),
    "stretch_mj_description": Description(
        Format.MJCF, tags={"mobile_manipulator"}
    ),
    "stretch_3_mj_description": Description(
        Format.MJCF, tags={"mobile_manipulator"}
    ),
    "talos_description": Description(Format.URDF, tags={"humanoid"}),
    "talos_mj_description": Description(Format.MJCF, tags={"humanoid"}),
    "tiago_description": Description(Format.URDF, tags={"mobile_manipulator"}),
    "tiago++_mj_description": Description(
        Format.MJCF, tags={"mobile_manipulator"}
    ),
    "toddlerbot_description": Description(Format.URDF, tags={"humanoid"}),
    "trifinger_edu_description": Description(
        Format.URDF, tags={"educational"}
    ),
    "upkie_description": Description(Format.URDF, tags={"biped", "wheeled"}),
    "ur10_description": Description(Format.URDF, tags={"arm"}),
    "ur10e_mj_description": Description(Format.MJCF, tags={"arm"}),
    "ur3_description": Description(Format.URDF, tags={"arm"}),
    "ur5_description": Description(Format.URDF, tags={"arm"}),
    "ur5e_mj_description": Description(Format.MJCF, tags={"arm"}),
    "valkyrie_description": Description(Format.URDF, tags={"humanoid"}),
    "viper_mj_description": Description(Format.MJCF, tags={"arm"}),
    "widow_mj_description": Description(Format.MJCF, tags={"arm"}),
    "wl_p311d_description": Description(Format.URDF, tags={"quadruped"}),
    "wl_p311e_description": Description(Format.URDF, tags={"quadruped"}),
    "xarm7_mj_description": Description(Format.MJCF, tags={"arm"}),
    "yam_description": Description(Format.URDF, tags={"arm"}),
    "yam_mj_description": Description(Format.MJCF, tags={"arm"}),
    "yumi_description": Description(Format.URDF, tags={"dual_arm"}),
    "z1_description": Description(Format.URDF, tags={"arm"}),
    "z1_mj_description": Description(Format.MJCF, tags={"arm"}),
}



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# SPDX-License-Identifier: Apache-2.0
# Copyright 2022 Giulio Romualdi
# Copyright 2023 Inria

"""Load a robot description in iDynTree."""

import os
from importlib import import_module  # type: ignore
from typing import List, Optional

import idyntree.swig as idyn

from .._package_dirs import get_package_dirs


def load_robot_description(
    description_name: str,
    joints_list: Optional[List[str]] = None,
    commit: Optional[str] = None,
) -> idyn.Model:
    """Load a robot description in iDynTree.

    Args:
        description_name: Name of the robot description.
        joints_list: Optional parameter containing the list of joints
            considered in the model. If empty, the model will contain all the
            joints specified in the URDF, otherwise a reduced model containing
            only the specified joints is created.
        commit: If specified, check out that commit from the cloned robot
            description repository.

    Returns:
        Identifier of the robot in iDynTree.

    Raises:
        ValueError:
            If the description is not URDF, or iDynTree is unable to load it.
    """
    if commit is not None:  # technical debt, see #31
        os.environ["ROBOT_DESCRIPTION_COMMIT"] = commit
    module = import_module(f"robot_descriptions.{description_name}")
    if commit is not None:
        os.environ.pop("ROBOT_DESCRIPTION_COMMIT", None)
    if not hasattr(module, "URDF_PATH"):
        raise ValueError(f"{description_name} is not a URDF description")

    model_loader = idyn.ModelLoader()

    if joints_list is None:
        if not model_loader.loadModelFromFile(
            module.URDF_PATH, "urdf", get_package_dirs(module)
        ):
            raise ValueError(
                f"Unable to load {description_name} with iDynTree"
            )
    else:
        if not model_loader.loadReducedModelFromFile(
            module.URDF_PATH, joints_list, "urdf", get_package_dirs(module)
        ):
            raise ValueError(
                f"Unable to load {description_name} with iDynTree"
            )

    robot = model_loader.model().copy()
    return robot



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# SPDX-License-Identifier: Apache-2.0
# Copyright 2025 Inria

"""
Show all humanoid robot descriptions using Pinocchio and MeshCat.
"""

try:
    import pinocchio as pin
    from pinocchio.visualize import MeshcatVisualizer
except ImportError as e:
    raise ImportError("Pinocchio not found, try `pip install pin`") from e

from robot_descriptions import DESCRIPTIONS
from robot_descriptions.loaders.pinocchio import load_robot_description

GRID_SPACING = 2.0  # [m]
GRID_SIZE = 5

if __name__ == "__main__":
    nb_humanoids = 0
    viewer = None
    visualizer = MeshcatVisualizer()

    for name, description in DESCRIPTIONS.items():
        if "humanoid" not in description.tags:
            continue
        if name == "talos_mj_description":
            # See https://github.com/stack-of-tasks/pinocchio/issues/2612
            continue
        if name == "op3_mj_description":
            # See https://github.com/stack-of-tasks/pinocchio/issues/2614
            continue
        robot = load_robot_description(
            name,
            root_joint=pin.JointModelFreeFlyer(),
        )
        robot.setVisualizer(visualizer)
        if viewer is None:
            robot.initViewer(open=True)
            viewer = robot.viewer
        robot.loadViewerModel()
        row = nb_humanoids // GRID_SIZE
        column = nb_humanoids % GRID_SIZE
        nb_humanoids += 1
        q = robot.q0.copy()
        q[0] = row * GRID_SPACING  # [m]
        q[1] = column * GRID_SPACING  # [m]
        robot.display(q)

    print(f"Displaying {nb_humanoids} humanoid robot descriptions")
    input("Press Enter to close MeshCat and terminate... ")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# SPDX-License-Identifier: Apache-2.0
# Copyright 2022 Stéphane Caron

import unittest

from robot_descriptions._descriptions import DESCRIPTIONS
from robot_descriptions.loaders.pinocchio import load_robot_description


class TestPinocchio(unittest.TestCase):

    """
    Check that all descriptions are loaded properly in Pinocchio.
    """

    @staticmethod
    def get_test_for_description(description: str):
        """
        Get test function for a given description.

        Args:
            description: Name of the description.

        Returns:
            Test function for that description.
        """

        def test(self):
            load_robot_description(description)

        return test


# Add a test function for each URDF description
for name, description in DESCRIPTIONS.items():
    if description.has_urdf:
        setattr(
            TestPinocchio,
            f"test_{name}",
            TestPinocchio.get_test_for_description(name),
        )



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# SPDX-License-Identifier: Apache-2.0
# Copyright 2022 Stéphane Caron

"""
Load a robot description selected from the command line in RoboMeshCat.

This example uses RoboMeshCat: https://github.com/petrikvladimir/RoboMeshCat
"""

import argparse

import numpy as np
import robomeshcat

from robot_descriptions.loaders.robomeshcat import load_robot_description

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument("name", help="name of the robot description")
    args = parser.parse_args()

    try:
        robot = load_robot_description(args.name)
    except ModuleNotFoundError:
        robot = load_robot_description(f"{args.name}_description")

    scene = robomeshcat.Scene()
    scene.add_robot(robot)

    with scene.animation(fps=1):
        scene.render()
        robot[0] = -0.5 * np.pi
        scene.render()

        for t in np.linspace(0, 2 * np.pi, 10):
            scene.camera_pos = np.array([np.sin(t), np.cos(t), 1.0])
            scene.render()

        scene.reset_camera()  # back to user interaction
        scene.render()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# SPDX-License-Identifier: Apache-2.0
# Copyright 2022 Stéphane Caron

from .test_clone_to_cache import TestCloneToCache
from .test_clone_to_directory import TestCloneToDirectory
from .test_descriptions import TestDescriptions
from .test_loaders import TestLoaders
from .test_progress_bar import TestProgressBar

__all__ = [
    "TestCloneToCache",
    "TestCloneToDirectory",
    "TestDescriptions",
    "TestLoaders",
    "TestProgressBar",
]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Fourier N1 description."""

from os import getenv as _getenv
from os import path as _path

from ._cache import clone_to_cache as _clone_to_cache

REPOSITORY_PATH: str = _clone_to_cache(
    "Wiki-GRx-Models-FourierN1",
    commit=_getenv("ROBOT_DESCRIPTION_COMMIT", None),
)

PACKAGE_PATH: str = _path.join(REPOSITORY_PATH, "N1")

URDF_PATH: str = _path.join(PACKAGE_PATH, "urdf", "N1_raw.urdf")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# SPDX-License-Identifier: Apache-2.0
# Copyright 2022 Stéphane Caron

"""
Load a robot description selected from the command line in Pinocchio.

This example requires Pinocchio, installed by e.g. `conda install pinocchio`.
"""

import argparse

from robot_descriptions.loaders.pinocchio import load_robot_description

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument("name", help="name of the robot description")
    args = parser.parse_args()

    try:
        robot = load_robot_description(args.name)
    except ModuleNotFoundError:
        robot = load_robot_description(f"{args.name}_description")

    print(f"Robot successfully loaded as {robot}")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# SPDX-License-Identifier: Apache-2.0
# Copyright 2022 Stéphane Caron

"""Valkyrie description."""

from os import getenv as _getenv
from os import path as _path

from ._cache import clone_to_cache as _clone_to_cache

REPOSITORY_PATH: str = _clone_to_cache(
    "nasa-urdf-robots",
    commit=_getenv("ROBOT_DESCRIPTION_COMMIT", None),
)

PACKAGE_PATH: str = _path.join(REPOSITORY_PATH, "val_description")

URDF_PATH: str = _path.join(
    PACKAGE_PATH, "model", "robots", "valkyrie_sim.urdf"
)

# Description-specific paths

URDF_PATH_A: str = _path.join(
    PACKAGE_PATH, "model", "robots", "valkyrie_A.urdf"
)

URDF_PATH_B: str = _path.join(
    PACKAGE_PATH, "model", "robots", "valkyrie_B.urdf"
)

URDF_PATH_C: str = _path.join(
    PACKAGE_PATH, "model", "robots", "valkyrie_C.urdf"
)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""GR-1 description."""

from os import getenv as _getenv
from os import path as _path

from ._cache import clone_to_cache as _clone_to_cache

REPOSITORY_PATH: str = _clone_to_cache(
    "Wiki-GRx-Models",
    commit=_getenv("ROBOT_DESCRIPTION_COMMIT", None),
)

PACKAGE_PATH: str = _path.join(REPOSITORY_PATH, "GRX", "GR1", "GR1T1")

URDF_PATH: str = _path.join(PACKAGE_PATH, "urdf", "GR1T1_nohand.urdf")

URDF_PATH_NO_HAND: str = _path.join(PACKAGE_PATH, "urdf", "GR1T1_no_hand.urdf")

URDF_PATH_FOURIER_HAND: str = _path.join(
    PACKAGE_PATH, "urdf", "GR1T1_fourier_hand_6dof.urdf"
)

URDF_PATH_INSPIRE_HAND: str = _path.join(
    PACKAGE_PATH, "urdf", "GR1T1_inspire_hand.urdf"
)

URDF_PATH_JAW: str = _path.join(PACKAGE_PATH, "GR1T1_jaw.urdf")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# SPDX-License-Identifier: Apache-2.0
# Copyright 2022 Stéphane Caron
# Copyright 2023 Inria

"""Load a robot description in MuJoCo."""

import os
from importlib import import_module  # type: ignore
from typing import Optional

import mujoco


def load_robot_description(
    description_name: str,
    commit: Optional[str] = None,
    variant: Optional[str] = None,
) -> mujoco.MjModel:
    """Load a robot description in MuJoCo.

    Args:
        description_name: Name of the robot description.
        commit: If specified, check out that commit from the cloned robot
            description repository.
        variant: If specified, load a specific variant of the robot model.

    Returns:
        Robot model for MuJoCo.
    """
    if commit is not None:  # technical debt, see #31
        os.environ["ROBOT_DESCRIPTION_COMMIT"] = commit
    module = import_module(f"robot_descriptions.{description_name}")
    if commit is not None:
        os.environ.pop("ROBOT_DESCRIPTION_COMMIT", None)
    if not hasattr(module, "MJCF_PATH"):
        raise ValueError(f"{description_name} is not an MJCF description")

    def load_model_from_path(path):
        try:
            return mujoco.MjModel.from_xml_path(path)
        except ValueError:
            print(f"{path} not found. Loading default robot model.")
            return None

    model_path = module.MJCF_PATH
    if variant is not None:
        path_parts = model_path.split(os.sep)
        path_parts[-1] = f"{variant}.xml"
        variant_path = os.sep.join(path_parts)
        model = load_model_from_path(variant_path)
        if model:
            return model
    return load_model_from_path(model_path)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# SPDX-License-Identifier: Apache-2.0
# Copyright 2022 Stéphane Caron

"""Atlas v4 description."""

from os import getenv as _getenv
from os import path as _path
from typing import List

from ._cache import clone_to_cache as _clone_to_cache

REPOSITORY_PATH: str = _clone_to_cache(
    "roboschool",
    commit=_getenv("ROBOT_DESCRIPTION_COMMIT", None),
)

PACKAGE_PATH: str = _path.join(
    REPOSITORY_PATH, "roboschool", "models_robot", "atlas_description"
)

MESH_PATHS: List[str] = [
    _path.join(PACKAGE_PATH, "meshes_v3"),
    _path.join(PACKAGE_PATH, "meshes_unplugged"),
    _path.join(
        REPOSITORY_PATH,
        "roboschool",
        "models_robot",
        "multisense_sl_description",
        "meshes",
    ),
]

URDF_PATH: str = _path.join(
    PACKAGE_PATH, "urdf", "atlas_v4_with_multisense.urdf"
)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# SPDX-License-Identifier: Apache-2.0
# Copyright 2022 Stéphane Caron

"""
Load a robot description selected from the command line in MuJoCo.

This example requires MuJoCo, which is installed by `pip install mujoco`.
"""

import argparse

from robot_descriptions.loaders.mujoco import load_robot_description

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument("name", help="name of the robot description")
    args = parser.parse_args()

    try:
        model = load_robot_description(args.name)
    except ModuleNotFoundError:
        model = load_robot_description(f"{args.name}_mj_description")

    print(f"Robot successfully loaded with model={model}")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# SPDX-License-Identifier: Apache-2.0
# Copyright 2022 Stéphane Caron
# Copyright 2023 Inria

"""Load a robot description in yourdfpy."""

import os
from importlib import import_module  # type: ignore
from typing import Optional

try:
    import yourdfpy
except ModuleNotFoundError as e:
    raise ModuleNotFoundError(
        "This feature requires 'yourdfpy', "
        "which can be installed by `pip install yourdfpy`"
    ) from e


def load_robot_description(
    description_name: str,
    commit: Optional[str] = None,
    **kwargs,
) -> yourdfpy.URDF:
    """Load a robot description in yourdfpy.

    Args:
        description_name: Name of the robot description.
        commit: If specified, check out that commit from the cloned robot
            description repository.
        kwargs: arguments passed to yourdfpy.URDF.load function, including:
            build_scene_graph: Whether to build a scene graph from visual
                elements.
            build_collision_scene_graph: Whether to build a scene graph from
                collision elements.
            load_meshes: Whether to load the meshes for the visual elements.
            load_collision_meshes: Whether to load the meshes for the collision
                elements.

    Returns:
        Robot model for yourdfpy.
    """
    if commit is not None:  # technical debt, see #31
        os.environ["ROBOT_DESCRIPTION_COMMIT"] = commit
    module = import_module(f"robot_descriptions.{description_name}")
    if commit is not None:
        os.environ.pop("ROBOT_DESCRIPTION_COMMIT", None)
    if not hasattr(module, "URDF_PATH"):
        raise ValueError(f"{description_name} is not a URDF description")

    return yourdfpy.URDF.load(
        module.URDF_PATH,
        mesh_dir=module.PACKAGE_PATH,
        **kwargs,
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# SPDX-License-Identifier: Apache-2.0
# Copyright 2022 Stéphane Caron

import os
import sys
import unittest
from importlib import import_module  # type: ignore

import git

from robot_descriptions._descriptions import DESCRIPTIONS


class TestDescriptions(unittest.TestCase):
    """Test fixture for all robot descriptions."""

    def test_all_descriptions(self):
        """Check all robot-description submodules."""
        for name, desc in DESCRIPTIONS.items():
            description = import_module(f"robot_descriptions.{name}")
            self.assertTrue(
                os.path.exists(description.REPOSITORY_PATH),
                f"Path {description.REPOSITORY_PATH} "
                f"does not exist in {description}",
            )
            self.assertTrue(
                os.path.exists(description.PACKAGE_PATH),
                f"Path {description.PACKAGE_PATH} "
                f"does not exist in {description}",
            )
            if desc.has_mjcf:
                self.assertTrue(hasattr(description, "MJCF_PATH"))
                self.assertTrue(
                    os.path.exists(description.MJCF_PATH),
                    f"MJCF path {description.MJCF_PATH} does not exist "
                    f"in {description}",
                )
            if desc.has_urdf:
                self.assertTrue(hasattr(description, "URDF_PATH"))
                self.assertTrue(
                    os.path.exists(description.URDF_PATH),
                    f"URDF path {description.URDF_PATH} does not exist "
                    f"in {description}",
                )

    def test_invalid_description_commit(self):
        invalid_commit = "foobar"
        os.environ["ROBOT_DESCRIPTION_COMMIT"] = invalid_commit
        self.assertEqual(os.getenv("ROBOT_DESCRIPTION_COMMIT"), invalid_commit)
        description_name = "robot_descriptions.sigmaban_description"
        with self.assertRaises(git.exc.GitCommandError):
            if description_name in sys.modules:
                del sys.modules[description_name]
            import_module(description_name)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# SPDX-License-Identifier: Apache-2.0
# Copyright 2022 Stéphane Caron
# Copyright 2023 Inria

"""Load a robot description in PyBullet."""

import os
from importlib import import_module  # type: ignore
from typing import Optional

import pybullet


def load_robot_description(
    description_name: str,
    commit: Optional[str] = None,
    **kwargs,
) -> int:
    """Load a robot description in PyBullet.

    Args:
        description_name: Name of the robot description.
        commit: If specified, check out that commit from the cloned robot
            description repository.
        kwargs: arguments passed to pybullet.loadURDF function, including:
            basePosition: 3D position of the base of the robot in world
                coordinates.
            baseOrientation: orientation in quaternion (xyzw) of the base of
                the robot in world coordinates.
            flags: int flags for the URDF loading in pybullet.
            useFixedBase: boolean indicating use a fix joint between world and
                robot base.
            physicsClientId: int indicating the pybullet client id.

    Returns:
        Integer identifier of the robot in PyBullet.
    """
    if commit is not None:  # technical debt, see #31
        os.environ["ROBOT_DESCRIPTION_COMMIT"] = commit
    module = import_module(f"robot_descriptions.{description_name}")
    if commit is not None:
        os.environ.pop("ROBOT_DESCRIPTION_COMMIT", None)
    if not hasattr(module, "URDF_PATH"):
        raise ValueError(f"{description_name} is not a URDF description")

    pybullet.setAdditionalSearchPath(module.PACKAGE_PATH)
    robot = pybullet.loadURDF(module.URDF_PATH, **kwargs)
    return robot



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# SPDX-License-Identifier: Apache-2.0
# Copyright 2022 Stéphane Caron

"""Import open source robot description as Python modules."""

from ._descriptions import DESCRIPTIONS

__version__ = "1.21.0"

__all__ = ["DESCRIPTIONS"]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# SPDX-License-Identifier: Apache-2.0
# Copyright 2024 Inria

"""
Load a robot description in a PyBullet physics simulation.

This example requires PyBullet, which can be installed with conda or pip.
"""

import argparse
import time

import pybullet
import pybullet_data
from robot_descriptions.loaders.pybullet import load_robot_description

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument("name", help="name of the robot description")
    parser.add_argument(
        "--duration",
        type=float,
        default=10.0,
        help="duration of the simulation in seconds",
    )
    parser.add_argument(
        "--timestep",
        type=float,
        default=0.01,
        help="duration of a simulation timestep in seconds",
    )
    args = parser.parse_args()

    pybullet.connect(pybullet.GUI)
    pybullet.setGravity(0.0, 0.0, -9.81)
    pybullet.setTimeStep(args.timestep)
    pybullet.configureDebugVisualizer(pybullet.COV_ENABLE_RENDERING, 0)
    pybullet.configureDebugVisualizer(pybullet.COV_ENABLE_SHADOWS, 0)
    pybullet.configureDebugVisualizer(pybullet.COV_ENABLE_GUI, 0)
    pybullet.setAdditionalSearchPath(pybullet_data.getDataPath())
    plane_id = pybullet.loadURDF("plane.urdf")

    try:
        robot = load_robot_description(args.name)
    except ModuleNotFoundError:
        robot = load_robot_description(f"{args.name}_description")

    initial_position = [0.0, 0.0, 1.0]
    initial_orientation = [0.0, 0.0, 0.0, 1.0]
    pybullet.resetBasePositionAndOrientation(
        robot, initial_position, initial_orientation
    )
    pybullet.configureDebugVisualizer(pybullet.COV_ENABLE_RENDERING, 1)

    for _ in range(int(args.duration / args.timestep)):
        pybullet.stepSimulation()
        time.sleep(args.timestep)

    pybullet.disconnect()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# SPDX-License-Identifier: Apache-2.0
# Copyright 2022 Stéphane Caron

import unittest

from robot_descriptions._cache import CloneProgressBar


class TestProgressBar(unittest.TestCase):
    def test_progress_bar(self):
        """
        Check progress bar.
        """
        bar = CloneProgressBar()
        bar.update(0, 42, 42)
        self.assertEqual(bar.progress.n, 42)
        self.assertEqual(bar.progress.total, 42)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# SPDX-License-Identifier: Apache-2.0
# Copyright 2023 Inria

"""
Display frames from a robot description.

This example relies on the following dependencies:

- `MeshCat <https://github.com/rdeits/meshcat-python>`_
- `meshcat-shapes <https://github.com/stephane-caron/meshcat-shapes>`_
- `Pinocchio <https://github.com/stack-of-tasks/pinocchio>`_
"""

import argparse
import time

import meshcat_shapes
import numpy as np
from meshcat import transformations
from pinocchio.visualize import MeshcatVisualizer

from robot_descriptions.loaders.pinocchio import load_robot_description

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument("name", help="name of the robot description")
    parser.add_argument(
        "--only",
        help="only display the frame with this name",
    )
    parser.add_argument(
        "--frame-scale",
        type=float,
        default=1.0,
        help="scaling factor applied to all frames",
    )
    args = parser.parse_args()

    try:
        robot = load_robot_description(args.name)
    except ModuleNotFoundError:
        robot = load_robot_description(f"{args.name}_description")

    robot.setVisualizer(MeshcatVisualizer())
    robot.initViewer(open=True)
    robot.loadViewerModel(color=[1.0, 1.0, 1.0, 0.3])
    robot.display(robot.q0)
    viewer = robot.viz.viewer

    for frame in robot.model.frames:
        if frame.name == "universe" or (args.only and frame.name != args.only):
            continue
        handle = viewer["pinocchio"]["visuals"][f"{frame.name}_0"]
        meshcat_shapes.frame(
            handle["frame"],
            axis_length=0.05 * args.frame_scale,
            axis_thickness=0.001 * args.frame_scale,
            opacity=0.8,
            origin_radius=0.005,
        )
        meshcat_shapes.textarea(
            handle["text"],
            text=frame.name,
            width=0.05,
            height=0.05,
            font_size=100,
        )
        Rx = transformations.rotation_matrix(0.5 * np.pi, [1.0, 0.0, 0.0])
        Rz = transformations.rotation_matrix(0.5 * np.pi, [0.0, 0.0, 1.0])
        trans = transformations.translation_matrix(
            [0.0, 0.0, 0.005 * args.frame_scale]
        )
        handle["text"].set_transform(trans @ Rz @ Rx)

    time.sleep(1.0)  # avoid terminating too fast



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# SPDX-License-Identifier: Apache-2.0

"""Ability Hand MJCF description."""

from os import getenv as _getenv
from os import path as _path

from ._cache import clone_to_cache as _clone_to_cache

REPOSITORY_PATH: str = _clone_to_cache(
    "ability-hand-api",
    commit=_getenv("ROBOT_DESCRIPTION_COMMIT", None),
)

PACKAGE_PATH: str = _path.join(
    REPOSITORY_PATH, "python", "ah_simulators", "mujoco_xml"
)

MJCF_PATH: str = _path.join(PACKAGE_PATH, "scene.xml")

# Description-specific paths

MJCF_PATH_LEFT_LARGE: str = _path.join(
    PACKAGE_PATH, "hands", "abh_left_large.xml"
)

MJCF_PATH_LEFT_SMALL: str = _path.join(
    PACKAGE_PATH, "hands", "abh_left_small.xml"
)

MJCF_PATH_RIGHT_LARGE: str = _path.join(
    PACKAGE_PATH, "hands", "abh_right_large.xml"
)

MJCF_PATH_RIGHT_SMALL: str = _path.join(
    PACKAGE_PATH, "hands", "abh_right_small.xml"
)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# SPDX-License-Identifier: Apache-2.0
# Copyright 2022 Stéphane Caron

"""
Show a robot description selected from the command line using PyBullet.

This example is equivalent to `python -m robot_descriptions show_in_pybullet`.
It requires PyBullet, which can be installed by `pip install pybullet`.
"""

import argparse

import pybullet

from robot_descriptions.loaders.pybullet import load_robot_description

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument("name", help="name of the robot description")
    args = parser.parse_args()

    pybullet.connect(pybullet.GUI)
    pybullet.configureDebugVisualizer(pybullet.COV_ENABLE_SHADOWS, 0)
    pybullet.configureDebugVisualizer(pybullet.COV_ENABLE_GUI, 0)

    try:
        robot = load_robot_description(args.name)
    except ModuleNotFoundError:
        robot = load_robot_description(f"{args.name}_description")

    input("Press Enter to close PyBullet and terminate... ")
    pybullet.disconnect()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# SPDX-License-Identifier: Apache-2.0
# Copyright 2022 Stéphane Caron
# Copyright 2023 Inria

"""Git utility functions to clone model repositories."""

import os
import shutil
from typing import Optional, Union

import tqdm
from git import (
    GitCommandError,
    InvalidGitRepositoryError,
    RemoteProgress,
    Repo,
)

from ._repositories import REPOSITORIES


class CloneProgressBar(RemoteProgress):
    """Progress bar when cloning."""

    def __init__(self):
        """Initialize progress bar."""
        super().__init__()
        self.progress = tqdm.tqdm()

    def update(
        self,
        op_code: int,
        cur_count: Union[str, float],
        max_count: Union[str, float, None] = None,
        message: str = "",
    ) -> None:
        """Update progress bar.

        Args:
            op_code: Integer that can be compared against Operation IDs and
                stage IDs.
            cur_count: Current item count.
            max_count: Maximum item count, or `None` if there is none.
            message: Unused here.
        """
        self.progress.total = max_count
        self.progress.n = cur_count
        self.progress.refresh()


def clone_to_directory(
    repo_url: str,
    target_dir: str,
    commit: Optional[str] = None,
) -> Repo:
    """Clone a git repository to a designated directory.

    Args:
        repo_url: URL to the git repository to clone.
        target_dir: Directory to clone the repository to.
        commit: Optional commit to check out after cloning.

    Returns:
        Cloned git repository.
    """
    clone = None
    if os.path.exists(target_dir):
        try:
            clone = Repo(target_dir)
        except InvalidGitRepositoryError:
            print(f"Repository at {target_dir} is invalid, recreating it...")
            shutil.rmtree(target_dir)
            clone = None

    if clone is None:
        print(f"Cloning {repo_url}...")
        os.makedirs(target_dir)
        progress_bar = CloneProgressBar()
        clone = Repo.clone_from(
            repo_url,
            target_dir,
            progress=progress_bar.update,
        )

    if commit is not None and commit != clone.head.object.hexsha:
        try:
            clone.git.checkout(commit)
        except GitCommandError:
            print(
                f"Commit {commit} not found, "
                "let's fetch origin and try again..."
            )
            clone.git.fetch("origin")
            clone.git.checkout(commit)
            print(f"Found commit {commit} successfully!")

    return clone


def clone_to_cache(description_name: str, commit: Optional[str] = None) -> str:
    """Get a local working directory cloned from a remote git repository.

    Args:
        description_name: Name of the robot description to clone.
        commit: If provided, checkout that specific commit of the description.

    Returns:
        Path to the resulting local working directory.

    Notes:
        By default, robot descriptions are cached to
        `~/.cache/robot_descriptions`. This behavior can be overriden by
        setting the `ROBOT_DESCRIPTIONS_CACHE` environment variable to an
        alternative path.
    """
    try:
        repository = REPOSITORIES[description_name]
    except KeyError as exn:
        raise ImportError(f"Unknown description: {description_name}") from exn

    cache_dir = os.path.expanduser(
        os.environ.get(
            "ROBOT_DESCRIPTIONS_CACHE",
            "~/.cache/robot_descriptions",
        )
    )

    cache_path = repository.cache_path
    if commit is not None:
        # Requirement: the last directory in the cache path is named after the
        # cache path (more precisely the package name) so that package:// URIs
        # work in frameworks that resolve them via directories
        cache_path = f"{cache_path}-{commit}/{cache_path}"
    target_dir = os.path.join(cache_dir, cache_path)
    clone = clone_to_directory(
        repository.url,
        target_dir,
        commit=repository.commit if commit is None else commit,
    )
    return str(clone.working_dir)


def clear_cache() -> None:
    """Clears the local cache where robot descriptions are stored."""
    cache_dir = os.path.expanduser(
        os.environ.get(
            "ROBOT_DESCRIPTIONS_CACHE",
            "~/.cache/robot_descriptions",
        )
    )
    shutil.rmtree(cache_dir, ignore_errors=True)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# SPDX-License-Identifier: Apache-2.0
# Copyright 2022 Stéphane Caron

"""
Show a robot description selected from the command line using MuJoCo.

This example is equivalent to `python -m robot_descriptions show_in_mujoco`. It
requires MuJoCo, which is installed by `pip install mujoco`, and the MuJoCo
viewer installed by `pip install mujoco-python-viewer`.
"""

import argparse

import mujoco

try:
    import mujoco_viewer
except ImportError as e:
    raise ImportError(
        "MuJoCo viewer not found, try `pip install mujoco-python-viewer`"
    ) from e

from robot_descriptions.loaders.mujoco import load_robot_description

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument("name", help="name of the robot description")
    args = parser.parse_args()

    try:
        model = load_robot_description(args.name)
    except ModuleNotFoundError:
        model = load_robot_description(f"{args.name}_mj_description")

    data = mujoco.MjData(model)
    viewer = mujoco_viewer.MujocoViewer(model, data)
    mujoco.mj_step(model, data)  # step at least once to load model in viewer
    while viewer.is_alive:
        viewer.render()
    viewer.close()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# SPDX-License-Identifier: Apache-2.0
# Copyright 2025 Inria

"""
Show a robot description selected from the command line in Candlewick.

This example requires Pinocchio, installed by e.g. `conda install pinocchio`,
and Candlewick: https://github.com/Simple-Robotics/candlewick.
"""

import argparse

try:
    from candlewick.multibody import Visualizer, VisualizerConfig
except ImportError as import_error:
    raise ImportError(
        "Candlewick not found, "
        "see https://github.com/Simple-Robotics/candlewick"
    ) from import_error

from robot_descriptions.loaders.pinocchio import load_robot_description

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument("name", help="name of the robot description")
    parser.add_argument("--width", help="window width in px", default=1600)
    parser.add_argument("--height", help="window height in px", default=900)
    args = parser.parse_args()

    try:
        robot = load_robot_description(args.name)
    except ModuleNotFoundError:
        robot = load_robot_description(f"{args.name}_description")

    config = VisualizerConfig()
    config.width = args.width
    config.height = args.height
    visualizer = Visualizer(config, robot.model, robot.visual_model)
    while not visualizer.shouldExit:
        visualizer.display(robot.q0)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# SPDX-License-Identifier: Apache-2.0
# Copyright 2022 Stéphane Caron

"""JAXON description."""

from os import getenv as _getenv
from os import path as _path

from ._cache import clone_to_cache as _clone_to_cache

REPOSITORY_PATH: str = _clone_to_cache(
    "jaxon_description",
    commit=_getenv("ROBOT_DESCRIPTION_COMMIT", None),
)

PACKAGE_PATH: str = _path.join(REPOSITORY_PATH)

URDF_PATH: str = _path.join(PACKAGE_PATH, "urdf", "jaxon_jvrc.urdf")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# SPDX-License-Identifier: Apache-2.0
# Copyright 2022 Stéphane Caron

"""Robonaut 2 description."""

from os import getenv as _getenv
from os import path as _path

from ._cache import clone_to_cache as _clone_to_cache

REPOSITORY_PATH: str = _clone_to_cache(
    "nasa-urdf-robots",
    commit=_getenv("ROBOT_DESCRIPTION_COMMIT", None),
)

PACKAGE_PATH: str = _path.join(REPOSITORY_PATH, "r2_description")

URDF_PATH: str = _path.join(
    PACKAGE_PATH, "robots", "r2c_sim_full_control.urdf.urdf"
)

# Description-specific paths

URDF_PATH_B: str = _path.join(PACKAGE_PATH, "robots", "r2b.urdf")
URDF_PATH_C1: str = _path.join(PACKAGE_PATH, "robots", "r2c1.urdf")
URDF_PATH_C5: str = _path.join(PACKAGE_PATH, "robots", "r2c5.urdf")
URDF_PATH_C6: str = _path.join(PACKAGE_PATH, "robots", "r2c6.urdf")

URDF_PATH_B_CONTROL: str = _path.join(
    PACKAGE_PATH, "robots", "r2b_control.urdf"
)
URDF_PATH_C1_CONTROL: str = _path.join(
    PACKAGE_PATH, "robots", "r2c1_control.urdf"
)
URDF_PATH_C5_CONTROL: str = _path.join(
    PACKAGE_PATH, "robots", "r2c5_control.urdf"
)
URDF_PATH_C6_CONTROL: str = _path.join(
    PACKAGE_PATH, "robots", "r2c6_control.urdf"
)

URDF_PATH_B_DYNAMICS: str = _path.join(
    PACKAGE_PATH, "robots", "r2b_dynamics.urdf"
)
URDF_PATH_C1_DYNAMICS: str = _path.join(
    PACKAGE_PATH, "robots", "r2c1_dynamics.urdf"
)
URDF_PATH_C5_DYNAMICS: str = _path.join(
    PACKAGE_PATH, "robots", "r2c5_dynamics.urdf"
)
URDF_PATH_C6_DYNAMICS: str = _path.join(
    PACKAGE_PATH, "robots", "r2c6_dynamics.urdf"
)

URDF_PATH_C_SIM_FULL_CONTROL: str = _path.join(
    PACKAGE_PATH, "robots", "r2c_sim_full_control.urdf"
)
URDF_PATH_C_SIM_FULL_DYNAMICS: str = _path.join(
    PACKAGE_PATH, "robots", "r2c_sim_full_dynamics.urdf"
)
URDF_PATH_C_SIM_LEGS_CONTROL: str = _path.join(
    PACKAGE_PATH, "robots", "r2c_sim_legs_control.urdf"
)
URDF_PATH_C_SIM_LEGS_DYNAMICS: str = _path.join(
    PACKAGE_PATH, "robots", "r2c_sim_legs_dynamics.urdf"
)
URDF_PATH_C_SIM_UPPERBODY_CONTROL: str = _path.join(
    PACKAGE_PATH, "robots", "r2c_sim_upperbody_control.urdf"
)
URDF_PATH_C_SIM_UPPERBODY_DYNAMICS: str = _path.join(
    PACKAGE_PATH, "robots", "r2c_sim_upperbody_dynamics.urdf"
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# SPDX-License-Identifier: Apache-2.0
# Copyright 2022 Stéphane Caron

import unittest

import pybullet

from robot_descriptions.loaders.idyntree import (
    load_robot_description as load_idyntree,
)
from robot_descriptions.loaders.mujoco import (
    load_robot_description as load_mujoco,
)
from robot_descriptions.loaders.pinocchio import (
    load_robot_description as load_pinocchio,
)
from robot_descriptions.loaders.pybullet import (
    load_robot_description as load_pybullet,
)
from robot_descriptions.loaders.robomeshcat import (
    load_robot_description as load_robomeshcat,
)
from robot_descriptions.loaders.yourdfpy import (
    load_robot_description as load_yourdfpy,
)


class TestLoaders(unittest.TestCase):
    """
    Test loaders.
    """

    upkie_description_commit = "98502d5b175c3d6b60b3cf475b7eeef9fd290c43"

    def test_mujoco(self):
        self.assertIsNotNone(load_mujoco("cassie_mj_description"))

    def test_pinocchio(self):
        self.assertIsNotNone(
            load_pinocchio(
                "upkie_description",
                commit=self.upkie_description_commit,
            )
        )

    def test_pybullet(self):
        pybullet.connect(pybullet.DIRECT)
        self.assertIsNotNone(
            load_pybullet(
                "upkie_description",
                commit=self.upkie_description_commit,
            )
        )
        pybullet.disconnect()

    def test_robomeshcat(self):
        self.assertIsNotNone(
            load_robomeshcat(
                "upkie_description",
                commit=self.upkie_description_commit,
            )
        )

    def test_yourdfpy(self):
        self.assertIsNotNone(
            load_yourdfpy(
                "upkie_description",
                commit=self.upkie_description_commit,
            )
        )

    def test_idyntree(self):
        self.assertIsNotNone(
            load_idyntree(
                "upkie_description",
                commit=self.upkie_description_commit,
            )
        )

    def test_cache_path_package_name(self):
        """Check a description with package:// URIs and a custom commit ID."""
        load_pinocchio(
            "draco3_description",
            commit="5afd19733d7b3e9f1135ba93e0aad90ed1a24cc7",
        )

    def test_load_with_commit_then_without(self):
        # https://github.com/robot-descriptions/robot_descriptions.py/issues/67
        load_pinocchio(
            "draco3_description",
            commit="5afd19733d7b3e9f1135ba93e0aad90ed1a24cc7",
        )
        load_pinocchio("baxter_description")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# SPDX-License-Identifier: Apache-2.0
# Copyright 2024 Inria

"""
We can load specific commit to pin the version of any robot description using
the `commit` keyword argument. In this example, we load a Universal Robots UR5e
from a specific commit of the MuJoCo Menagerie repository that distributes it.
This example requires MuJoCo, which is installed by `pip install mujoco`.

See also:
    https://github.com/robot-descriptions/robot_descriptions.py/discussions/86
"""

from robot_descriptions.loaders.mujoco import load_robot_description

if __name__ == "__main__":
    model = load_robot_description(
        "ur5e_mj_description",
        commit="cc3868ec21bae324e6dd3b01ea185b6e6f0fa172",
    )
    print(f"UR5e successfully loaded in {model=}")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# SPDX-License-Identifier: Apache-2.0
# Copyright 2022 Stéphane Caron

import os
import tempfile
import unittest

import git

from robot_descriptions._cache import clone_to_cache, clear_cache
from robot_descriptions._repositories import REPOSITORIES


class TestCloneToCache(unittest.TestCase):
    def test_clone_to_cache_found(self):
        """
        Test clone_to_cache on a valid repository.
        """
        working_dir_1 = clone_to_cache("upkie_description")
        working_dir_2 = clone_to_cache("upkie_description")
        self.assertEqual(working_dir_1, working_dir_2)

    def test_clone_to_cache_not_found(self):
        """
        Test clone_to_cache on an invalid repository.
        """
        with self.assertRaises(ImportError):
            clone_to_cache("foo")

    def test_clone_to_cache_right_commit(self):
        """
        Check that clone_to_cache clones the repository at the right commit.
        """
        description_name = "simple_humanoid_description"
        repo = git.Repo(clone_to_cache(description_name))
        commit = str(list(repo.iter_commits(max_count=1))[0])
        self.assertEqual(commit, REPOSITORIES[description_name].commit)

    def test_cache_creation(self):
        """
        Check that clone_to_cache creates directory if needed.
        """
        description_name = "simple_humanoid_description"
        repo = REPOSITORIES[description_name]

        with tempfile.TemporaryDirectory() as tmp_dir:
            os.environ["ROBOT_DESCRIPTIONS_CACHE"] = tmp_dir
            clone_to_cache(description_name)
            self.assertTrue(
                os.path.exists(os.path.join(tmp_dir, repo.cache_path))
            )
            del os.environ["ROBOT_DESCRIPTIONS_CACHE"]

    def test_clone_invalid_commit(self):
        """Invalid commit raises an exception."""
        with self.assertRaises(git.exc.GitCommandError):
            clone_to_cache("simple_humanoid_description", commit="foobar")

    def test_clone_valid_commit(self):
        description_name = "simple_humanoid_description"
        commit = "0e488ee4708155a71b2a92d05305a9186b543593"
        repository_path = clone_to_cache(description_name, commit)
        self.assertTrue(commit in repository_path)

    def test_clone_with_commit_then_without(self):
        clear_cache()
        clone_to_cache(
            "upkie_description",
            commit="a2c820054a7572603875def478f21376165c125e",
        )
        clone_to_cache("draco3_description")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# SPDX-License-Identifier: Apache-2.0
# Copyright 2022 Stéphane Caron

"""
Show a robot description selected from the command line using MeshCat.

This example is equivalent to `python -m robot_descriptions show_in_meshcat`.
It requires Pinocchio, installed by e.g. `conda install pinocchio`, and
MeshCat, installed by e.g. `conda install meshcat-python`.
"""

import argparse

try:
    from pinocchio.visualize import MeshcatVisualizer
except ImportError as e:
    raise ImportError("Pinocchio not found, try `pip install pin`") from e

from robot_descriptions.loaders.pinocchio import load_robot_description

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument("name", help="name of the robot description")
    args = parser.parse_args()

    try:
        robot = load_robot_description(args.name)
    except ModuleNotFoundError:
        robot = load_robot_description(f"{args.name}_description")

    robot.setVisualizer(MeshcatVisualizer())
    robot.initViewer(open=True)
    robot.loadViewerModel()
    robot.display(robot.q0)

    input("Press Enter to close MeshCat and terminate... ")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# SPDX-License-Identifier: Apache-2.0
# Copyright 2022 Stéphane Caron

import os
import tempfile
import unittest

import git

from robot_descriptions._cache import clone_to_directory
from robot_descriptions._repositories import REPOSITORIES


class TestCloneToDirectory(unittest.TestCase):
    def test_clone_to_directory(self):
        """Check cloning on a dummy repository."""
        with tempfile.TemporaryDirectory() as tmp_dir:
            repo_name = "test_repo"
            repo_path = os.path.join(tmp_dir, repo_name)
            clone_dir = os.path.join(tmp_dir, "clone")
            empty_file = os.path.join(repo_path, "foo.bar")
            repo = git.Repo.init(repo_path)
            open(empty_file, "wb").close()
            repo.index.add([empty_file])
            repo.index.commit("Test initial commit")

            # Clone the repository for the first time
            clone_1 = clone_to_directory(repo_path, clone_dir)
            self.assertTrue(clone_1.common_dir.startswith(tmp_dir))
            self.assertTrue(clone_1.working_dir.endswith("clone"))

            # Cloning again, should recover the existing repo
            clone_2 = clone_to_directory(repo_path, clone_dir)
            self.assertEqual(
                str(clone_1.active_branch), str(clone_2.active_branch)
            )
            self.assertTrue(clone_1.common_dir, clone_2.common_dir)
            self.assertTrue(clone_1.working_dir, clone_2.working_dir)

    def test_clone_to_directory_newer_commit(self):
        """Repository is pulled when a newer commit is configured."""
        description_name = "simple_humanoid_description"
        repo_params = REPOSITORIES[description_name]

        def get_commit(repo):
            return repo.git.rev_parse("HEAD")

        with tempfile.TemporaryDirectory() as tmp_dir:
            repo_dir = os.path.join(tmp_dir, "test")
            repo = clone_to_directory(
                repo_params.url, repo_dir, commit=repo_params.commit
            )
            self.assertFalse(repo.bare)
            self.assertEqual(get_commit(repo), repo_params.commit)
            repo.git.reset("HEAD~1", "--hard")
            self.assertNotEqual(get_commit(repo), repo_params.commit)
            repo_bis = clone_to_directory(
                repo_params.url, repo_dir, commit=repo_params.commit
            )
            self.assertEqual(get_commit(repo), repo_params.commit)
            self.assertEqual(get_commit(repo_bis), repo_params.commit)

    def test_clone_to_invalid_directory(self):
        """Cloning to an empty (invalid) git repo recreates it."""
        description_name = "simple_humanoid_description"
        repo_params = REPOSITORIES[description_name]
        with tempfile.TemporaryDirectory() as tmp_dir:
            repo_dir = os.path.join(tmp_dir, "test")
            os.mkdir(repo_dir)
            self.assertEqual(len(os.listdir(repo_dir)), 0)
            clone_to_directory(repo_params.url, repo_dir)
            self.assertGreater(len(os.listdir(repo_dir)), 0)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# SPDX-License-Identifier: Apache-2.0
# Copyright 2022 Stéphane Caron

"""Git utility functions to clone model repositories."""

from dataclasses import dataclass
from typing import Dict


@dataclass
class Repository:
    """Remote git repository.

    Attributes:
        cache_path: Path to clone the repository to in the local cache.
        commit: Commit ID or tag to checkout after cloning.
        url: URL to the remote git repository.
    """

    cache_path: str
    commit: str
    url: str


REPOSITORIES: Dict[str, Repository] = {
    "ability-hand-api": Repository(
        url="https://github.com/psyonicinc/ability-hand-api.git",
        commit="89407424edfc22faceaedcd7c3ea2b7947cbbb2c",
        cache_path="ability_hand_api",
    ),
    "anymal_b_simple_description": Repository(
        url="https://github.com/ANYbotics/anymal_b_simple_description.git",
        commit="988b5df22b84761bdf08111b1c2ccc883793f456",
        cache_path="anymal_b_simple_description",
    ),
    "anymal_c_simple_description": Repository(
        url="https://github.com/ANYbotics/anymal_c_simple_description.git",
        commit="f160b8f7fed840c47a6febe8e2bc78b32bf43a68",
        cache_path="anymal_c_simple_description",
    ),
    "anymal_d_simple_description": Repository(
        url="https://github.com/ANYbotics/anymal_d_simple_description.git",
        commit="6adc14720aab583613975e5a9d6d4fa3cfcdd081",
        cache_path="anymal_d_simple_description",
    ),
    "bambot": Repository(
        url="https://github.com/timqian/bambot.git",
        commit="04d902653794f9f72eeabb09ec90a9af8e397c5b",
        cache_path="bambot",
    ),
    "baxter_common": Repository(
        url="https://github.com/RethinkRobotics/baxter_common.git",
        commit="6c4b0f375fe4e356a3b12df26ef7c0d5e58df86e",  # v1.2.0
        cache_path="baxter_common",
    ),
    "berkeley_humanoid_description": Repository(
        url="https://github.com/HybridRobotics/berkeley_humanoid_description",
        commit="d0d13d3f81d795480e25ed1910eaf83d5f0a1d0b",
        cache_path="berkeley_humanoid_description",
    ),
    "bhand_model": Repository(
        url="https://github.com/jhu-lcsr-attic/bhand_model.git",
        commit="937f4186d6458bd682a7dae825fb6f4efe56ec69",
        cache_path="bhand_model",
    ),
    "booster_gym": Repository(
        url="https://github.com/BoosterRobotics/booster_gym.git",
        commit="687a33d08b08875fe45dc8d91b54db83766df8b9",
        cache_path="booster_gym",
    ),
    "bullet3": Repository(
        url="https://github.com/bulletphysics/bullet3.git",
        commit="7dee3436e747958e7088dfdcea0e4ae031ce619e",
        cache_path="bullet3",
    ),
    "bxi_robot_models": Repository(
        url="https://github.com/bxirobotics/robot_models.git",
        commit="eabe24ce937f8e633077a163b883e92e8996c36e",
        cache_path="bxi_robot_models",
    ),
    "cassie_description": Repository(
        url="https://github.com/robot-descriptions/cassie_description",
        commit="81a2d8bbd77201cc974afb127adda4e2857a6dbf",
        cache_path="cassie_description",
    ),
    "cassie_mj_description": Repository(
        url="https://github.com/rohanpsingh/cassie_mj_description.git",
        commit="fcc3775453e4da6797cd4eacd3d8321d9906755a",
        cache_path="cassie_mj_description",
    ),
    "differentiable-robot-model": Repository(
        url="https://github.com/facebookresearch/differentiable-robot-model",
        commit="d7bd1b3b8ef1d6dabe9b68474a622185c510e112",
        cache_path="differentiable-robot-model",
    ),
    "draco3_description": Repository(
        url="https://github.com/shbang91/draco3_description.git",
        commit="5afd19733d7b3e9f1135ba93e0aad90ed1a24cc7",
        cache_path="draco3_description",
    ),
    "drake": Repository(
        url="https://github.com/RobotLocomotion/drake.git",
        commit="7abea0556ede980a5077fe1a8cfbae59b57c7c27",
        cache_path="drake",
    ),
    "drake_models": Repository(
        url="https://github.com/RobotLocomotion/models.git",
        # NB: from 9b274a2570ddefc4140e4b98bcf248003289b870 onward, obj meshes
        # for the iiwa14_description are converted to glTF in this repo, which
        # PyBullet does not load ("invalid mesh filename extension 'gltf'")
        commit="d0fe1a427a6bd39040ff3a77aaf6ddcc4d62a8fe",
        cache_path="drake_models",
    ),
    "eDO_description": Repository(
        url="https://github.com/ianathompson/eDO_description.git",
        commit="17b3f92f834746106d6a4befaab8eeab3ac248e6",
        cache_path="edo_sim",  # match package name
    ),
    "ergocub-software": Repository(
        url="https://github.com/icub-tech-iit/ergocub-software.git",
        commit="v0.7.7",
        cache_path="ergocub-software",
    ),
    "example-robot-data": Repository(
        url="https://github.com/Gepetto/example-robot-data.git",
        commit="d0d9098d752014aec3725b07766962acf06c5418",  # v4.0.9
        cache_path="example-robot-data",
    ),
    "fanuc_m710ic_description": Repository(
        url="https://github.com/"
        "robot-descriptions/fanuc_m710ic_description.git",
        commit="d12af44559cd7e46f7afd513237f159f82f8402e",
        cache_path="fanuc_m710ic_description",
    ),
    "GingerURDF": Repository(
        url="https://github.com/Rayckey/GingerURDF.git",
        commit="6a1307cd0ee2b77c82f8839cdce3a2e2eed2bd8f",
        cache_path="gingerurdf",  # match package name
    ),
    "gym-pybullet-drones": Repository(
        url="https://github.com/utiasDSL/gym-pybullet-drones.git",
        commit="2c2470c2043eb4cf474d9af2681c9fa10f2d0e2f",
        cache_path="gym-pybullet-drones",
    ),
    "halodi-robot-models": Repository(
        url="https://github.com/Halodi/halodi-robot-models.git",
        commit="ba9e7c8cdbd63e20fc6526dbbea1b91c102fb820",
        cache_path="halodi-robot-models",
    ),
    "i2rt": Repository(
        url="https://github.com/i2rt-robotics/i2rt.git",
        commit="7809b5b17227162d265f922e2e10598c0e214322",
        cache_path="i2rt",
    ),
    "icub-models": Repository(
        url="https://github.com/robotology/icub-models.git",
        commit="v1.25.0",
        cache_path="icub-models",
    ),
    "jaxon_description": Repository(
        url="https://github.com/robot-descriptions/jaxon_description.git",
        commit="4a0cb7a4a737864312f8d6e3f89823a741539bfc",
        cache_path="jaxon_description",
    ),
    "jvrc_description": Repository(
        url="https://github.com/jrl-umi3218/jvrc_description.git",
        commit="9ff8efbc7043459a8f0892662bd030d8020fb682",
        cache_path="jvrc_description",
    ),
    "jvrc_mj_description": Repository(
        url="https://github.com/isri-aist/jvrc_mj_description.git",
        commit="0f0ce7daefdd66c54e0909a6bf2c22154844f5f3",
        cache_path="jvrc_mj_description",
    ),
    "LEAP_Hand_Sim": Repository(
        url="https://github.com/leap-hand/LEAP_Hand_Sim.git",
        commit="150bc3d4b61fd6619193ba5a8ef209f3609ced89",
        cache_path="LEAP_Hand_Sim",
    ),
    "kinova_mj_description": Repository(
        url="https://github.com/mathieu-celerier/kinova_mj_description.git",
        commit="cee556b0a438e6904634a90826d4e8d2e005cd1f",
        cache_path="kinova_mj_description",
    ),
    "limxdynamics_robot-description": Repository(
        url="https://github.com/limxdynamics/robot-description.git",
        commit="a097533372a08298d45af391cbdfc2fd2dc3da6f",
        # TODO: allow cache path with sub-directories; here we want the
        # "robot_description" as last directory name, to match the package path
        # in URDF descriptions so that Bullet loads them successfully
        cache_path="robot_description",
    ),
    "mini_cheetah_urdf": Repository(
        url="https://github.com/Derek-TH-Wang/mini_cheetah_urdf.git",
        commit="1988bceb26e81f28594a16e7d5e6abe5cbb27ace",
        cache_path="mini_cheetah_urdf",
    ),
    "mujoco": Repository(
        url="https://github.com/google-deepmind/mujoco.git",
        commit="ad0dc0de5e10a075a2c65be629e9a8d557d383a6",
        cache_path="mujoco",
    ),
    "mujoco_menagerie": Repository(
        url="https://github.com/deepmind/mujoco_menagerie.git",
        commit="c1503a62496b64222c64ff65dd652d461a5b064e",
        cache_path="mujoco_menagerie",
    ),
    "nao_robot": Repository(
        url="https://github.com/ros-naoqi/nao_robot.git",
        commit="67476469a1371b00b17538eb6ea336367ece7d44",
        cache_path="nao_robot",
    ),
    "nasa-urdf-robots": Repository(
        url="https://github.com/gkjohnson/nasa-urdf-robots.git",
        commit="54cdeb1dbfb529b79ae3185a53e24fce26e1b74b",
        cache_path="nasa-urdf-robots",
    ),
    "onshape-to-robot-examples": Repository(
        url="https://github.com/Rhoban/onshape-to-robot-examples.git",
        commit="911abb069c781e4c717c10643b975f55f7a64fe8",
        cache_path="onshape-to-robot-examples",
    ),
    "pepper_description": Repository(
        url="https://github.com/jrl-umi3218/pepper_description.git",
        commit="cd9715bb5df7ad57445d953db7b1924255305944",
        cache_path="pepper_description",
    ),
    "Piper_ros": Repository(
        url="https://github.com/agilexrobotics/Piper_ros.git",
        commit="f2ec6a67e1f404bcb478529e89861ccdf43fa298",
        cache_path="Piper_ros",
    ),
    "poppy_ergo_jr_description": Repository(
        url="https://github.com/poppy-project/poppy_ergo_jr_description.git",
        commit="7eb32bd385afa11dea5e6a6b6a4a86a0243aaa2b",
        cache_path="poppy_ergo_jr_description",
    ),
    "poppy_torso_description": Repository(
        url="https://github.com/poppy-project/poppy_torso_description.git",
        commit="6beeec3d76fb72b7548cce7c73aad722f8884522",
        cache_path="poppy_torso_description",
    ),
    "rby1_description": Repository(
        url="https://github.com/uynitsuj/rby1_description.git",
        commit="e4c07203aa0a0d1b6b3b39da105cb00a77e2bc72",
        cache_path="rby1_description",
    ),
    "reachy_description": Repository(
        url="https://github.com/aubrune/reachy_description.git",
        commit="release-1.0.0",
        cache_path="reachy_description",
    ),
    "rhea_description": Repository(
        url="https://github.com/G-Levine/rhea_description.git",
        commit="1dc0f1abcf51b5d8a8f7ff8a548399ff0df1414f",
        cache_path="rhea_description",
    ),
    "robot-assets": Repository(
        url="https://github.com/ankurhanda/robot-assets.git",
        commit="12f1a3c89c9975194551afaed0dfae1e09fdb27c",
        cache_path="robot-assets",
    ),
    "roboschool": Repository(
        url="https://github.com/openai/roboschool.git",
        commit="1.0.49",
        cache_path="roboschool",
    ),
    "robotiq_arg85_description": Repository(
        url="https://github.com/a-price/robotiq_arg85_description.git",
        commit="a65190bdbb0666609fe7e8c3bb17341e09e81625",
        cache_path="robotiq_arg85_description",
    ),
    "romeo_robot": Repository(
        url="https://github.com/ros-aldebaran/romeo_robot.git",
        commit="0.1.5",
        cache_path="romeo_robot",
    ),
    "ros2_kortex": Repository(
        url="https://github.com/Kinovarobotics/ros2_kortex.git",
        commit="fb633aacf9c1c85d61a96c1099b5afa3d533e5a3",
        cache_path="ros2_kortex",
    ),
    "rtmros_nextage": Repository(
        url="https://github.com/tork-a/rtmros_nextage.git",
        commit="ac270fb969fa54abeb6863f9b388a9e20c1f14e0",
        cache_path="rtmros_nextage",
    ),
    "sigmaban_urdf": Repository(
        url="https://github.com/Rhoban/sigmaban_urdf.git",
        commit="d5d023fd35800d00d7647000bce8602617a4960d",
        cache_path="sigmaban_urdf",
    ),
    "simple_humanoid_description": Repository(
        url="https://github.com/laas/simple_humanoid_description.git",
        commit="4e859aed7df3c29954c9cca2a1ecb94069f7cfce",
        cache_path="simple_humanoid_description",
    ),
    "SO-ARM100": Repository(
        url="https://github.com/TheRobotStudio/SO-ARM100.git",
        commit="63eede5a636e548eb8f2854e558bd343c21db9f7",
        cache_path="SO-ARM100",
    ),
    "spryped": Repository(
        url="https://github.com/bbokser/spryped.git",
        commit="f360a6b78667a4d97c86cad465ef8f4c9512462b",
        cache_path="spryped",
    ),
    "stretch_description": Repository(
        url="https://github.com/robot-descriptions/stretch_description.git",
        commit="4b838429fe4c5d9f2937efe698444bd68968f376",
        cache_path="stretch_description",
    ),
    "talos-data": Repository(
        url="https://github.com/stack-of-tasks/talos-data.git",
        commit="77169405d6a48a5d3f3f75eb014209f375ff23b6",  # v2.0.0
        cache_path="talos_data",  # match package name
    ),
    "toddlerbot": Repository(
        url="https://github.com/hshi74/toddlerbot.git",
        commit="067f9dc4f50143e36334877b9395b9c5c29ee30c",
        cache_path="toddlerbot",
    ),
    "unitree_mujoco": Repository(
        url="https://github.com/unitreerobotics/unitree_mujoco.git",
        commit="f3300ff1bf0ab9efbea0162717353480d9b05d73",
        cache_path="unitree_mujoco",
    ),
    "unitree_ros": Repository(
        url="https://github.com/unitreerobotics/unitree_ros.git",
        commit="267182b8521c8d6a631bab1fe63836873237a525",
        cache_path="unitree_ros",
    ),
    "upkie_description": Repository(
        url="https://github.com/upkie/upkie_description.git",
        commit="19a91ce69cab6742c613cab104986e3f8a18d6a5",
        cache_path="upkie_description",
    ),
    "skydio_x2_description": Repository(
        url="https://github.com/lvjonok/skydio_x2_description.git",
        commit="9a6a057a055babaf47119fac42c361fffc189128",
        cache_path="skydio_x2_description",
    ),
    "Wiki-GRx-Models": Repository(
        url="https://github.com/FFTAI/Wiki-GRx-Models.git",
        commit="351245ac8fa4bf6f4b0c41556e1e6976a438bcef",
        cache_path="Wiki-GRx-Models",
    ),
    "Wiki-GRx-Models-FourierN1": Repository(
        url="https://github.com/FFTAI/Wiki-GRx-Models.git",
        commit="f8e683f00d1d99deb882deb9dfce6030095b466a",  # FourierN1 branch
        cache_path="Wiki-GRx-Models-FourierN1",
    ),
}



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# SPDX-License-Identifier: Apache-2.0
# Copyright 2022 Stéphane Caron

"""List or show robot descriptions from the command line."""

import argparse
from importlib import import_module  # type: ignore
from typing import List

from robot_descriptions._descriptions import DESCRIPTIONS


def positive_float(value) -> float:
    """Convert a value to float and check that it is positive.

    Args:
        value: Value to convert.

    Returns:
        Converted value.

    Raises:
        ArgumentTypeError: if the value is not positive.
    """
    float_value = float(value)
    if float_value <= 0.0:
        raise argparse.ArgumentTypeError(
            f"Duration {value} is not a positive number"
        )
    return float_value


def get_argument_parser() -> argparse.ArgumentParser:
    """Parse command-line arguments.

    Returns:
        Argument parser.
    """
    parser = argparse.ArgumentParser(description=__doc__)
    subparsers = parser.add_subparsers(title="subcommands", dest="subcmd")

    # list --------------------------------------------------------------------
    subparsers.add_parser(
        "list",
        help="list all available robot descriptions",
    )

    # pull ----------------------------------------------------------------
    parser_pull = subparsers.add_parser(
        "pull",
        help="load given robot description from the web and save it in cache",
    )
    parser_pull.add_argument(
        "name",
        help="name of the robot description",
    )

    # show_in_meshcat --------------------------------------------------------
    parser_meshcat = subparsers.add_parser(
        "show_in_meshcat",
        help="load and display a given robot description in Meshcat",
    )
    parser_meshcat.add_argument(
        "name",
        help="name of the robot description",
    )

    # show_in_mujoco --------------------------------------------------------
    parser_mujoco = subparsers.add_parser(
        "show_in_mujoco",
        help="load and display a given robot description in mujoco",
    )
    parser_mujoco.add_argument(
        "name",
        help="name of the robot description",
    )

    # show_in_pybullet --------------------------------------------------------
    parser_pybullet = subparsers.add_parser(
        "show_in_pybullet",
        help="load and display a given robot description in pybullet",
    )
    parser_pybullet.add_argument(
        "name",
        help="name of the robot description",
    )

    # show_in_yourdfpy --------------------------------------------------------
    parser_yourdfpy = subparsers.add_parser(
        "show_in_yourdfpy",
        help="load and display a given robot description with yourdfpy",
    )
    parser_yourdfpy.add_argument(
        "name",
        help="name of the robot description",
    )
    parser_yourdfpy.add_argument(
        "-c",
        "--configuration",
        nargs="+",
        type=float,
        help="configuration of the visualized robot description",
    )
    parser_yourdfpy.add_argument(
        "--collision",
        action="store_true",
        help="use collision geometry for the visualized robot description",
    )

    # animate -----------------------------------------------------------------
    parser_animate = subparsers.add_parser(
        "animate",
        help=(
            "animate a robot description by interpolating "
            "each joint between its limits"
        ),
    )
    parser_animate.add_argument(
        "name",
        help="name of the robot description",
    )

    return parser


def list_descriptions():
    """List descriptions to the standard output."""
    for name in sorted(list(DESCRIPTIONS)):
        desc = DESCRIPTIONS[name]
        formats = ("URDF" if desc.has_urdf else "") + (
            "MJCF" if desc.has_mjcf else ""
        )
        print(f"- {name} [{formats}]")


def pull(name: str) -> None:
    """Pull a robot description from the web and save it in cache.

    Args:
        name: Name of the robot description.
    """
    try:
        module = import_module(f"robot_descriptions.{name}")
    except ModuleNotFoundError:
        module = import_module(f"robot_descriptions.{name}_description")
    if hasattr(module, "URDF_PATH"):
        print(module.URDF_PATH)
    elif hasattr(module, "MJCF_PATH"):
        print(module.MJCF_PATH)


def show_in_meshcat(name: str) -> None:
    """Show a robot description in MeshCat.

    Args:
        name: Name of the robot description.
    """
    try:
        from pinocchio.visualize import MeshcatVisualizer
    except ImportError as e:
        raise ImportError(
            "Pinocchio not found, try for instance `conda install pinocchio`"
        ) from e

    from robot_descriptions.loaders.pinocchio import load_robot_description

    try:
        robot = load_robot_description(name)
    except ModuleNotFoundError:
        robot = load_robot_description(f"{name}_description")

    robot.setVisualizer(MeshcatVisualizer())
    robot.initViewer(open=True)
    robot.loadViewerModel()
    robot.display(robot.q0)

    input("Press Enter to close MeshCat and terminate... ")


def show_in_mujoco(name: str) -> None:
    """Show a robot description in MuJoCo.

    Args:
        name: Name of the robot description.
    """
    import mujoco

    try:
        import mujoco_viewer
    except ImportError as e:
        raise ImportError(
            "MuJoCo viewer not found, try `pip install mujoco-python-viewer`"
        ) from e

    from robot_descriptions.loaders.mujoco import load_robot_description

    try:
        model = load_robot_description(name)
    except ModuleNotFoundError:
        model = load_robot_description(f"{name}_mj_description")

    data = mujoco.MjData(model)
    viewer = mujoco_viewer.MujocoViewer(model, data)
    mujoco.mj_step(model, data)  # step at least once to load model in viewer
    while viewer.is_alive:
        viewer.render()
    viewer.close()


def show_in_pybullet(name: str) -> None:
    """Show a robot description in PyBullet.

    Args:
        name: Name of the robot description.
    """
    import pybullet

    from robot_descriptions.loaders.pybullet import load_robot_description

    pybullet.connect(pybullet.GUI)
    pybullet.configureDebugVisualizer(pybullet.COV_ENABLE_SHADOWS, 0)
    pybullet.configureDebugVisualizer(pybullet.COV_ENABLE_GUI, 0)

    try:
        load_robot_description(name)
    except ModuleNotFoundError:
        load_robot_description(f"{name}_description")

    input("Press Enter to close PyBullet and terminate... ")
    pybullet.disconnect()


def show_in_yourdfpy(
    name: str,
    configuration: List[float],
    collision: bool,
) -> None:
    """Load and display a given robot description.

    Args:
        name: Name of the robot description.
        configuration: Optional robot configuration.
        collision: Use collision rather than visualization geometry.
    """
    try:
        module = import_module(f"robot_descriptions.{name}")
    except ModuleNotFoundError:
        module = import_module(f"robot_descriptions.{name}_description")
    if not hasattr(module, "URDF_PATH"):
        raise ValueError(
            "This command only works with URDF descriptions, use "
            "`show_in_mujoco` for MJCF descriptions"
        )

    try:
        yourdfpy = import_module("yourdfpy")
    except ModuleNotFoundError as exc:
        raise ValueError(
            "This command requires yourdfpy. You can installed it by: "
            "`pip install yourdfpy`"
        ) from exc

    if collision:
        robot = yourdfpy.URDF.load(
            module.URDF_PATH,
            build_collision_scene_graph=True,
            load_collision_meshes=True,
        )
    else:
        robot = yourdfpy.URDF.load(module.URDF_PATH)

    if configuration:
        robot.update_cfg(configuration)
    robot.show(
        collision_geometry=collision,
    )


def animate(name: str) -> None:
    """Show how to call yourdfpy to animate a given robot description.

    Args:
        name: Name of the robot description.
    """
    try:
        module = import_module(f"robot_descriptions.{name}")
    except ModuleNotFoundError:
        module = import_module(f"robot_descriptions.{name}_description")

    if not hasattr(module, "URDF_PATH"):
        raise ValueError(
            "animation is only available for URDF descriptions, "
            "check out the `show_in_mujoco.py` example for MJCF"
        )

    print(
        "Check out yourdfpy more URDF features like animation or checking "
        "collision meshes. In this instance, the yourdfpy command is:\n\n"
        f"\tyourdfpy {module.URDF_PATH} --animate\n"
    )


def main(argv=None):
    """Command line entry point."""
    parser = get_argument_parser()
    args = parser.parse_args(argv)
    if args.subcmd == "list":
        list_descriptions()
    elif args.subcmd == "pull":
        pull(args.name)
    elif args.subcmd == "show_in_meshcat":
        show_in_meshcat(args.name)
    elif args.subcmd == "show_in_mujoco":
        show_in_mujoco(args.name)
    elif args.subcmd == "show_in_pybullet":
        show_in_pybullet(args.name)
    elif args.subcmd == "show_in_yourdfpy":
        show_in_yourdfpy(args.name, args.configuration, args.collision)
    elif args.subcmd == "animate":
        animate(args.name)
    else:  # no subcommand
        parser.print_help()


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# SPDX-License-Identifier: Apache-2.0
# Copyright 2022 Stéphane Caron

import unittest

import pybullet

from robot_descriptions._descriptions import DESCRIPTIONS

from robot_descriptions.loaders.pybullet import load_robot_description


class TestPyBullet(unittest.TestCase):

    """
    Check that all descriptions are loaded properly in PyBullet.
    """

    def setUp(self):
        """
        Start PyBullet simulation.
        """
        pybullet.connect(pybullet.DIRECT)

    def tearDown(self):
        """
        Stop PyBullet simulation.
        """
        pybullet.disconnect()

    def test_value_error_when_no_urdf(self):
        """
        Test exception raised when a description has no URDF_PATH.
        """
        with self.assertRaises(ValueError):
            load_robot_description("_empty_description")

    @staticmethod
    def get_test_for_description(description: str):
        """
        Get test function for a given description.

        Args:
            description: Name of the description.

        Returns:
            Test function for that description.
        """

        def test(self):
            load_robot_description(description)

        return test


for name, description in DESCRIPTIONS.items():
    if description.has_urdf:
        setattr(
            TestPyBullet,
            f"test_{name}",
            TestPyBullet.get_test_for_description(name),
        )



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# SPDX-License-Identifier: Apache-2.0
# Copyright 2022 Stéphane Caron
# Copyright 2023 Inria

"""Load a robot description in `RoboMeshCat`_.

.. _RoboMeshCat: https://github.com/petrikvladimir/RoboMeshCat
"""

import os
from importlib import import_module  # type: ignore
from typing import Optional

import robomeshcat

from .pinocchio import get_package_dirs


def load_robot_description(
    description_name: str,
    commit: Optional[str] = None,
) -> robomeshcat.Robot:
    """Load a robot description in RoboMeshCat.

    Args:
        description_name: Name of the robot description.
        commit: If specified, check out that commit from the cloned robot
            description repository.

    Returns:
        Robot model for RoboMeshCat.
    """
    if commit is not None:  # technical debt, see #31
        os.environ["ROBOT_DESCRIPTION_COMMIT"] = commit
    module = import_module(f"robot_descriptions.{description_name}")
    if commit is not None:
        os.environ.pop("ROBOT_DESCRIPTION_COMMIT", None)
    robot = robomeshcat.Robot(
        urdf_path=module.URDF_PATH,
        mesh_folder_path=get_package_dirs(module),
    )
    return robot



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# SPDX-License-Identifier: Apache-2.0
# Copyright 2025 Inria

"""Ability Hand description."""

from os import getenv as _getenv
from os import path as _path

from ._cache import clone_to_cache as _clone_to_cache

REPOSITORY_PATH: str = _clone_to_cache(
    "ability-hand-api",
    commit=_getenv("ROBOT_DESCRIPTION_COMMIT", None),
)

PACKAGE_PATH: str = _path.join(
    REPOSITORY_PATH,
    "URDF",
)

URDF_PATH: str = _path.join(PACKAGE_PATH, "ability_hand_left_large.urdf")

# Description-specific paths

URDF_PATH_LEFT_LARGE: str = _path.join(
    PACKAGE_PATH, "ability_hand_left_large.urdf"
)

URDF_PATH_LEFT_SMALL: str = _path.join(
    PACKAGE_PATH, "ability_hand_left_small.urdf"
)

URDF_PATH_RIGHT_LARGE: str = _path.join(
    PACKAGE_PATH, "ability_hand_right_large.urdf"
)

URDF_PATH_RIGHT_SMALL: str = _path.join(
    PACKAGE_PATH, "ability_hand_right_small.urdf"
)
